% Example_12_1
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
% Frequency response of FIR QMF bank
% Johnston's coefficients

clear all
close all

 B1=[-0.006443977,0.02745539,-0.00758164,-0.0913825,0.09808522,0.4807962];% filter
 length N=12

 %B1=[0.002329266,-0.005182978,-0.002273145,0.01354012,-0.0006504669,-0.02755195,...
 %    0.01004621,0.05088162,-0.03464143,-0.09987885,0.12464520,0.4686479];% filter 
% length N=24

h0 = [B1,fliplr(B1)];      % Generating the lowpass filter H0(z)
figure (1)
subplot(2,1,1)
stem(0:length(h0)-1,h0);title('Impulse response of low pass filter')
axis([0,11,-0.6,0.6])
xlabel('n')
% Generate the complementary highpass filter

for k = 1:length(h0)      % Generating the highpass filter H1(z)
   h1(k) = ((-1)^k)*h0(k);
end
figure (1)
subplot(2,1,2)
stem(0:length(h1)-1,h1);title('Impulse response of high pass filter')
axis([0,11,-0.6,0.6])
xlabel('n')
% Compute the gain responses of the two filters
[H1z,w]=freqz(h0,1,256);
H0=abs(H1z); g1=20*log10(H0);
[H2z,w]=freqz(h1,1,256);
H1=abs(H2z); g2=20*log10(H1);
% Plot the gain responses of the two filters
figure (2)
subplot(4,1,1:3)
plot(w/pi,g1,'-',w/pi,g2,'-b'),grid
%xlabel('Normalized frequency')
ylabel('Gain, dB')
axis([0,1,-60,2])
%f=16/256;
% x=sin(2*pi*f*(0:511)); % Input signal
% x=0:1/512:1-1/512;

% Compute the sum of the square magnitude responses
ver=H0.^2+H1.^2;
figure (2)
subplot(4,1,4)
plot(w/pi,10*log10(ver))
xlabel('Normalized frequency \omega/\pi'), ylabel('Gain, dB')
% title('Amplitude distortion, dB')

% Test signal
x = [zeros(size(1:100)),ones(size(101:250)),zeros(size(251:511))]; % Test signal
% x=x+0.1*randn(size(0:511));
% Polyphase decomposition
e0 = h0(1:2:length(h0));         % Polyphase component E_0(z)
e1 = h0(2:2:length(h0));         % Polyphase component E_1(z)

% Analysis part
x0 = x(1:2:length(x))/2;   % Down-sampling, branch E_0
x1 = [0,x(2:2:length(x)-1)]/2;  % Down-sampling, branch E_1
v00 = filter(e0,1,x0);         % Filtering with E_0(z)
v11 = filter(e1,1,x1);         % Filtering with E_1(z)
v0 = v00 + v11; % Output of the lowpass filter
v1 = v00 - v11; % Output of the highpass filter

% Synthesis part
w00 = v0 + v1; % Input to polyphase component E_1(z), synthesis part
w11 = v0 - v1; % Input to polyphase component E_0(z), synthesis part
u00 = 2*filter(e1,1,w00);   % Filtering with E_1(z)
u11 = 2*filter(e0,1,w11);   % Filtering with E_0(z)	
y0 = zeros(size(1:512));
y0(1:2:512) = u00; % Upsampling, branch E_0
y1 = zeros(size(1:512));
y1(1:2:512) = u11; % Upsampling, branch E_1
y0 = [0,y0(1:511)]; % Inserting the delay z^(-1)
y = 2*(y0 + y1); % Reconstructed signal

figure (3)
subplot(7,1,1:2)
plot(1:511,x), ylabel('x[n]')
axis([0,512,-0.1,1.1])
subplot(7,1,3:4)
plot(0:255,v0),ylabel('v_0[n]')
axis([0,256,-0.1,1.1])
%title('Signal at the output of the low-pass analysis filter')
subplot(7,1,5)
plot(0:255,v1),,ylabel('v_1[n]')
axis([0,256,-0.1,0.1])
%title('Signal at the output of the high-pass analysis filter')
subplot(7,1,6:7)
plot(0:511,y), xlabel('Time index n'), ylabel('y[n]')
%title('Reconstructed signal')
axis([0,512,-0.1,1.1])