% Example_12_2
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
%    N = Butterworth halfband filter order 
clear all, close all
N=input('N= ')             % Selecting the Butterworth filter order
[z,p,k]=butter(N,0.5);     % Butterworth filter design
c=abs(p).^2;               % Squared moduli of the filter poles
g=sort(c(1:2:length(c)));  
bet0=g(2:2:length(g));    % Selecting beta coefficients for A0(z)
bet1=g(3:2:length(g));    % Selecting beta coefficients for A1(z)

f=50/256;



% Composing the allpass branches A_0(z) and A_1(z)
P0=poly(-bet0) % Allpass branch A_0(z)=Q0(z)/P0(z)
Q0=fliplr(P0)  % Allpass branch A_0(z)=Q0(z)/P0(z)
P1=poly(-bet1) % Allpass branch A_1(z)=Q1(z)/P1(z)
Q1=fliplr(P1) % Allpass branch A_1(z)=Q1(z)/P1(z)

TB = conv(upsample(Q0,2),upsample(Q1,2)); 
TB = [0,TB];
TA = conv(upsample(P0,2),upsample(P1,2));
[gd,f] = grpdelay(TB,TA,512,2); 
figure (1)
subplot(2,1,1)
plot(f,gd), grid, xlabel('\omega/\pi'),ylabel('Group delay, samples')
%axis([0,1,0,12])

%x=[zeros(size(0:100)),ones(size(101:800)),zeros(size(801:1022))];
x = [zeros(size(1:100)),ones(size(101:250)),zeros(size(251:511))]; % Generating the test signal

% Analysis bank
xx=x/2;  % Scaling the input signal by 1/2
x0=xx(1:2:length(xx));  % Down-sampling, branch A0
x1=[0,xx(2:2:length(xx))]; % Down-sampling, branch A1
v00=filter(Q0,P0,x0); % Filtering with A_0(z)
v11=filter(Q1,P1,x1); % Filtering with A_1(z)
v0=v00+v11;  % Lowpass output
v1=v00-v11;  % Highpass output


% Synthesis bank
w00=v0+v1; % Input to A_0(z)
w11=v0-v1; % Input to B_0(z)
u00=filter(Q1,P1,w00); % Filtering with A_1(z)
u11=filter(Q0,P0,w11); % Filtering with A_0(z)
y0=zeros(size(1:512));
y0(1:2:512)=u00;      % Up-sampling, branch A_1
y1=zeros(size(1:512));
y1(1:2:512)=u11;  % Up-sampling, branch B_1
y0=[0,y0(1:511)];  %  Inserting delay z^(-1)

y=y0+y1;  % Reconstructed signal

figure (2)
subplot(4,1,1:3)
B=k*poly(z); A=poly(p);     % Lowpass filter H0(z)
[H0,f]=freqz(B,A,256,2);    % Lowpass filter frequency response
plot(f,abs(H0))
hold on
BB=k*poly(-z);   AA = A;           % Highpass filter H1(z)
[H1,f]=freqz(BB,AA,256,2);     % Highpass filter frequency response
plot(f,abs(H1)),ylabel('Magnitude'), grid
axis([0,1,0,1.1])

ver=abs(H0).^2+abs(H1).^2;
figure (2)
subplot(4,1,4)
plot(f,10*log10(ver))
%plot(f,ver)
xlabel('\omega/\pi'), ylabel('Gain, dB')
% title('Amplitude distortion, dB')

figure (3)
subplot(7,1,1:2)
plot(1:511,x), ylabel('x[n]')
axis([0,512,-0.3,1.3])
subplot(7,1,3:4)
plot(0:255,v0),ylabel('v_0[n]')
axis([0,256,-0.3,1.3])
%title('Signal at the output of the low-pass analysis filter')
subplot(7,1,5)
plot(0:255,v1),,ylabel('v_1[n]')
axis([0,256,-0.3,0.3])
%title('Signal at the output of the high-pass analysis filter')
subplot(7,1,6:7)
plot(0:511,y), xlabel('Time index n'), ylabel('y[n]')
%title('Reconstructed signal')
axis([0,512,-0.3,1.3])